;################################################################### LIBGB.INC
; Purpose.....:	LIBG: BIOS and BDOS calls, included by LIBG.MAC
;#############################################################################
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ gBxxxxxx: BDOS calls
		IF	ugBDOS
;==================================================================== gB....()
; Purpose.....: Call BDOS functions
; Input.......: All registers set as required for the function to call
; Registers...: C destroyed plus the ones which return a value
; Requires....: gcBDOS
;=============================================================================

;------------- define which registers return a value for a particular function
gcBBTA		EQU	6		; register A
gcBBTH		EQU	7		;          HL

;------------------------------------------------------- function entry points
gBCONI::	LD	C,01H		; console input
		SET	gcBBTA,C	; A=ascii character 
		JP	gBCALL

gBCONO::	LD	C,02H		; console output
		JP	gBCALL

gBRDRI::	LD	C,03H		; reader input
		SET	gcBBTA,C	; A=ascii character
		JP	gBCALL

gBPUNO::	LD	C,04H		; punch output
		JP	gBCALL

gBLSTO::	LD	C,05H		; list output
		JP	gBCALL

gBDCIO::	LD	C,06H		; direct console I/O
		SET	gcBBTA,C	; A=char or status
		JP	gBCALL

gBGIOB::	LD	C,07H		; get I/O byte
		SET	gcBBTA,C	; A=I/O byte value
		JP	gBCALL

gBSIOB::	LD	C,08H		; set I/O byte
		JP	gBCALL

gBSTRO::	LD	C,09H		; string output
		JR	gBCALL

gBRCOB::	LD	C,0AH		; read console buffer
		JR	gBCALL

gBCONS::	LD	C,0BH		; get console status
		SET	gcBBTA,C	; A=console status
		JR	gBCALL

gBGVER::	LD	C,0CH		; return version number
		SET	gcBBTH,C	; HL=version number
		JR	gBCALL

gBRDSK::	LD	C,0DH		; reset disk system
		JR	gBCALL

gBSDSK::	LD	C,0EH		; select disk
		JR	gBCALL

gBFOPN::	LD	C,0FH		; file open 
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFCLS::	LD	C,010H		; file close
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFSFI::	LD	C,011H		; search for first
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFSNE::	LD	C,012H		; search for next
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFDEL::	LD	C,013H		; delete file
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFRDS::	LD	C,014H		; file read sequential
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFWRS::	LD	C,015H		; file write sequential
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFMAK::	LD	C,016H		; make file
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBFREN::	LD	C,017H		; rename file
		SET	gcBBTA,C	; A=directory code
		JR	gBCALL

gBRLV::		LD	C,018H		; return login vector
		SET	gcBBTH,C	; HL=login vector
		JR	gBCALL

gBRCD::		LD	C,019H		; return current disk
		SET	gcBBTA,C	; A=current disk
		JR	gBCALL

gBFDMA::	LD	C,01AH		; set DMA address
		JR	gBCALL

gBFRDR::	LD	C,021H		; read random
		SET	gcBBTA,C	; A=return code
		JR	gBCALL

gBFWRR::	LD	C,022H		; write random
		SET	gcBBTA,C	; A=return code
		JR	gBCALL

gBFCFS::	LD	C,023H		; compute file size
		JR	gBCALL

gBFSRR::	LD	C,024H		; set random record
		JR	gBCALL

gBDRST::	LD	C,025H		; reset drive
		SET	gcBBTA,C	; A=always 0
		JR	gBCALL

gBFWRZ::	LD	C,026H		; write random with zero fill
		SET	gcBBTA,C	; a=return code

;------------------------------------------------------------ do the BDOS call
gBCALL:		PUSH	BC
		PUSH	DE
		PUSH	AF
		PUSH	HL

		PUSH	BC		; save return register bits
		RES	gcBBTA,C	; and clear them
		RES	gcBBTH,C

		CALL	gcBDOS

;----------------------------------- update the registers which return a value
		POP	BC		; recall return register bits
gBRHL:		BIT	gcBBTH,C	; does HL return a value?
		JR	Z,gBRA		; no
		INC	SP		; replace HL on stack with
		INC	SP		; return value
		PUSH	HL
	
gBRA:		BIT	gcBBTA,C	; does A return a value?
		JR	Z,gBX		; no
		INC	SP		; replace AF on stack with
		INC	SP		; return value
		INC	SP
		INC	SP
		PUSH	AF
		DEC	SP
		DEC	SP
	
gBX:		POP	HL
		POP	AF
		POP	DE
		POP	BC
		RET
;================================================================ eoF gB....()
		ENDIF
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++ gIxxxxxx: BIOS calls
		IF	ugBIOS
;===================================================================== gBIOS()
; Purpose.....: Call BIOS function
; Input.......: A:  Offset from BIOS entry point, one of the below gIxxx
;		DE: Input to requested BIOS routine
;               BC: Input to requested BIOS routine
; Result......: As defined in CP/M BIOS
; Registers...: None saved
; Requires....: Nothing
;=============================================================================

;--------------------------------------------------------- BIOS function calls
gcIWBT		EQU	0		; WBOOT:   warm boot, reload CCP
gcICOS		EQU	3		; CONST:   console status
gcICOI		EQU	6		; CONIN:   console input
gcICOO		EQU	9		; CONOUT:  console output
gcILST		EQU	12		; LIST:    printer output
gcIPUN		EQU	15		; PUNCH:   punch output
gcIRDR		EQU	18		; READER:  reader input
gcIHME		EQU	21		; HOME:    move disk head to track 0
gcISDR		EQU	24		; SELDSK:  select disk drive
gcISTR		EQU	27		; SETTRK:  set track number
gcISSE		EQU	30		; SETSEC:  set sector number
gcISDM		EQU	33		; SETDMA:  set DMA address
gcIRDS		EQU	36		; READ:    read a sector
gcIWRS		EQU	39		; WRITE:   write a sector
gcILSS		EQU	42		; LISTST:  get status of list device
gcISCT		EQU	45		; SECTRAN: sector translation table
	
gBIOS::		PUSH	DE		; save ev. param
		LD	E,A		; E=offset
		LD	D,00H		; clear hibyte of offset
		LD	HL,(1)		; get BIOS entry address
		ADD	HL,DE		; add requested offset
		POP	DE		; recall ev. param
		JP	(HL)		; jump to BIOS, returns with RET
;================================================================= eoF gBIOS()
		ENDIF
;############################################################### EOF LIBGB.INC
======================== eoF gBIOS()
		ENDIF
;################################################