;################################################################### LIBGD.INC
; Purpose.....:	LIBG: Disk functions. Included by LIBG.MAC
;#############################################################################

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++ file i/o using BDOS
		IF	ugDFR OR ugDFW
;================================================= file read/write error codes
		PUBLIC	gcDFWD
		PUBLIC	gcDFWF
		PUBLIC	gcDFRN
		PUBLIC	gcDFRE
gcDFRN		EQU	0FFH		; file not found error
gcDFRE		EQU	0FEH		; end of file error
gcDFWD		EQU	0FDH		; out of directory space
gcDFWF		EQU	0FCH		; disk full/io error
;=============================================================================
		ENDIF			; ugDFR OR ugDFW
		IF	ugDFCS		
;===================================================================== gDFCS()
; Purpose.....: Clear and setup FCB for file operations
; Input.......: BC: address of filename in FCB format
;		DE: address of FCB to use
;		A:  drive to use, 0=default, 1=A:, 2=B ...
; Registers...: All saved
; Requires....: Nothing
; Labels......:	gDFCSx: g)enp D)isk F)ileC)ontrolblock S)etup
;=============================================================================
gDFCS::		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF

;-------------------------- clear out FCB, might hold info from a previous use
		EX	DE,HL		; HL= FCB address
		PUSH	HL		; save it
		LD	A,gcFCBZ	; length of FCB

gDFCBI:		LD	(HL),0		; fill whole FCB with 0
		INC	HL
		DEC	A
		JR	NZ,gDFCBI
		POP	HL		; recall FCB address

;------------------------------------------ copy drive to FCB, set closed flag
		POP	AF		; recall drive
		PUSH	AF		; correct stack

		LD	(HL),A		; save drive to FCB
		SET	7,(HL)		; set file closed flag	
		INC	HL		; HL points to FCB's filename

;-------------------------------------------------------- copy filename to FCB
		LD	E,11		; length of filename		
gDFCSL:		LD	A,(BC)		; get filename character
		LD	(HL),A		; store in FCB
		INC	HL
		INC	BC
		DEC	E
		JR	NZ,gDFCSL	; repeat until done

		POP	AF
		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================= eoF gDFCS()
		ENDIF			; ugDFCS
		IF	ugDFCC		
;===================================================================== gDFCC()
; Purpose.....: Clear FCB, leave filename in place
; Input.......: DE: FCB address
; Registers...: All saved
; Requires....: Nothing
; Labels......:	gDFCCx: g)enp D)isk F)ileC)ontrolblock C)lear
;=============================================================================
gDFCC::		PUSH	HL
		PUSH	DE
		PUSH	BC

		EX	DE,HL		; HL= FCB address
		SET	7,(HL)		; set FCB to closed
		LD	DE,12		; skip filename
		ADD	HL,DE
		LD	B,gcFCBZ-12	; length of FCB

gDFCCI:		LD	(HL),0		; fill rest of FCB with 0
		INC	HL
		DJNZ	gDFCCI

		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================= eoF gDFCC()
		ENDIF			; ugDFCC
		IF	ugDFO
;====================================================================== gDFO()
; Purpose.....: Open or create file if it's not already open
; Input.......: DE: FCB address
;		A:  0= open in overwrite mode (file is deleted first)
;		    1= open in append mode (filesize is computed)
;		    this only applies if open is in write mode (CY=1)	
;		CY: 0= do not create if it does not exist (read mode)
;		    1= create it (write mode)
; Output......: A:  error code in case of error, 0 otherwise
;		CY: 0= file opened/created/already open
;		    1= file does not exist/directory full
; Registers...: AF destroyed
; Requires....: gBFOPN(), gBFDEL(), gBFMAK()
; Labels......: gDFCOx: g)genp D)isk F)ile O)pen
;=============================================================================
gDFO::		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF		; save create/append flags

		EX	DE,HL		; HL= FCB address
		BIT	7,(HL)		; bit7=1 if file closed
		JR	Z,gDFOS		; file is open, exit with CY=0

		RES	7,(HL)		; clear the closed flag
		EX	DE,HL		; DE= FCB address
		CALL	gBFOPN		; BDOS open file
		LD	B,A		; save file found status

		POP	AF		; recall create/append flags
		PUSH	AF		; correct stack
		BIT	0,A		; A0=1: append to file
		JR	NZ,gDFOA	; append, create or calc file size
		JR	C,gDFOD		; CY=1: delete/create file
		INC	B		; read: file present?
		LD	A,gcDFRN	; read: error code file not found
		JR	Z,gDFOE		; error: file not found
		JR	gDFOS		; read, file present, ok

gDFOA:		INC	B		; append: file present?
		JR	Z,gDFOC		; no, create it
		CALL	gBFCFS		; yes, calc file size, update FCB
		JR	gDFOS		; exit with success

gDFOD:		INC	B		; B=FF if file not present
		JR	Z,gDFOC		; skip delete if not present
		CALL	gBFDEL		; delete file if present
		CALL	gDFCC		; clean up FCB
		EX	DE,HL		; reset file-closed flag
		RES	7,(HL)		; set by gDFCC 
		EX	DE,HL		; DE= FCB address

gDFOC:		CALL	gBFMAK		; create file
		INC	A		; FF= no directory space
		LD	A,gcDFWD	; error code no directory space
		JR	NZ,gDFOS	; return with success

gDFOE:		CALL	gDFCC		; error, reset FCB
		SCF			; signal error
		JR	gDFOX		; and exit
 		
gDFOS:		XOR	A		; CY=0 to signal success

gDFOX:		INC	SP		; correct initial PUSH AF
		INC	SP		; A and F return values
		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================== eoF gDFO()
		ENDIF			; ugDFO
		IF	ugDFCL
;===================================================================== gDFCL()
; Purpose.....: Close a file if not already closed, clear FCB except for 
;		filename
; Input.......: DE: FCB address
; Registers...: AF destroyed
; Requires....: gBFCLS()
; Labels......: gDFCLx: g)enp D)isk F)ile C)L)ose
;=============================================================================
gDFCL::		PUSH	DE
		PUSH	BC

		LD	A,(DE)		; check file closed flag
		BIT	7,A		; is it set?
		JR	NZ,gDFCLS	; yes, nothing to do

		CALL	gBFCLS		; BDOS: close file
		INC	A		; FF= file not found
		SCF			; preset to error
		LD	A,gcDFRN	; preset to not found	
		CALL	gDFCC		; clean-up FCB
		JR	Z,gDFCLX	; file not found, CY=1

gDFCLS:		XOR	A		; success exit, CY=0		

gDFCLX:		POP	BC
		POP	DE
		RET
;================================================================= eoF gDFCL()
		ENDIF			; ugDFCL
		IF	ugDFD
;====================================================================== gDFD()
; Purpose.....: Delete a file, clear FCB except for filename
; Input.......: DE: FCB address
; Output......: A:  error code, 0 if no error
;		CY: 1= file not found
; Registers...: AF destroyed
; Requires....: gDFCS(), gBFCLS(), gBFDEL()
; Labels......: gDFDxx: g)enp D)isk F)ile D)elete
;=============================================================================
gDFD::		PUSH	DE
		PUSH	BC

		LD	A,(DE)		; clear file closed flag
		RES	7,A
		LD	(DE),A

		CALL	gBFDEL		; BDOS: delete file
		INC	A		; FF= file not found
		SCF			; preset to error
		LD	A,gcDFRN	; preset to not found	
		CALL	gDFCC		; clean-up FCB
		JR	Z,gDFDX		; file not found, CY=1

		XOR	A		; success, CY=0		

gDFDX:		POP	BC
		POP	DE
		RET
;================================================================== eoF gDFD()
		ENDIF			; ugDFD
		IF	ugDFE
;====================================================================== gDFE()
; Purpose.....: Check if a file exists. Uses it's own FCB
; Input.......: BC: Address of filename
;		A:  Drive, 0=current, 1=A, 2=B ...
; Output......: CY: 0= file exists
;		    1= it does not
; Registers...: AF destroyed
; Requires....: gDFCS(), gBFOPN()
; Labels......: gDFExx: g)genp D)isk F)ile E)xists
;=============================================================================
gDFE::		PUSH	HL
		PUSH	DE
		PUSH	BC

		PUSH	AF		; save drive
		LD	DE,gvDFEF	; DE= local FCB address
		CALL	gDFCS		; setup FCB
		POP	AF		; recall drive
		LD	(DE),A		; write to FCB

		CALL	gBFOPN		; BDOS: open file
		INC	A		; FF= file not found
		SCF			; preset to not found
		JR	Z,gDFEX		; not found, exit with CY=1
		XOR	A		; found, exit with CY=0	

gDFEX:		POP	BC
		POP	DE
		POP	HL
		RET

;------------------------------------------------------------------- variables
gvDFEF:		DS	gcFCBZ		; local FCB
;================================================================== eoF gDFE()
		ENDIF			; ugDFE
		IF	ugDFR
;====================================================================== gDFR()
; Purpose.....: Read a file from disk. If the file is already open, read
;		from last position.
; Input.......: HL: buffer from address
;               BC: buffer to address
;               DE: FCB address
; Output......: A:  error code in case of error, 0 otherwise
;		CY: 0= success
;		    1= disk read error or file not found
; Registers...: AF destroyed
; Requires....: gBFDMA(), gBFOPN(), gBFRDS(), gUHLDE()
; Labels......: gDFRxx: g)enp D)isk F)ile R)ead
;=============================================================================
gDFR::		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	(gvDFRC),DE	; store FCB address

;-------------------------------------------------------- try to open the file
		XOR	A		; do not create file
		CALL	gDFO		; open the file
		JR	C,gDFRE		; file not found

;--------------------------------------------------------------- read the file
gDFRL:		POP	DE		; DE= end address
		PUSH	DE		; keep stack happy
		CALL	gUHLDE		; check if done
		JR	Z,gDFRQ		; equal, done, exit
		JR	C,gDFRR		; not done, go for next block
		JR	gDFRQ		; done

gDFRR:		EX	DE,HL		; DE= store address
		CALL	gBFDMA		; set DMA address to next block
		EX	DE,HL		; HL= store address
		LD	DE,(gvDFRC)	; DE= FCB address
		CALL	gBFRDS		; BDOS read sequential
		OR	A		; is read result zero?
		LD	A,gcDFRE	; signal read error (eof)
		JR	NZ,gDFRE	; no, error

		LD	DE,128		; DE= block size
		ADD	HL,DE		; add it to store address
		JR	gDFRL		; next block

;--------------------------------------------------------------- error handler
gDFRE:		SCF			; signal error to caller
		JR	gDFRX		; and exit

gDFRQ:		XOR	A		; CY=0, signal no error
	
gDFRX:		POP	BC
		POP	DE
		POP	HL
		RET

;------------------------------------------------------------------- variables
gvDFRC:		DW	0		; FCB address
;================================================================== eoF gDFR()
		ENDIF			; ugDFR
		IF	ugDFW
;====================================================================== gDFW()
; Purpose.....: Write a file to disk
; Input.......: HL: from address
;               BC: to address
;               DE: FCB address
;		CY: 0= overwrite
;		    1= append
; Output......: A:  error code in case of error, 0 otherwise 
;		CY: 0= success
;		    1= some error
; Registers...: AF destroyed
; Requires....: gDFO(), gBFDMA(), gBFWRR()
; Labels......: gDFWxx: g)enp D)isk F)ile W)rite
;=============================================================================
gDFW::		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	(gvDFWC),DE	; save FCB address

;------------------------------------------------------------ open/create file
		RLA			; A0=1: append, 0: overwrite
		SCF			; create file if does not exist
		CALL	gDFO		; open/create file
		JR	C,gDFWX		; some error, exit

;-------------------------------------------------------------- write the file
gDFWL:		POP	DE		; DE= end address
		PUSH	DE		; keep stack happy
		CALL	gUHLDE		; compare HL and DE
		JR	Z,gDFWQ		; equal, done, exit
		JR	C,gDFWW		; not done, go for next block
		JR	gDFWQ		; done, exit

gDFWW:		EX	DE,HL		; DE= read address
		CALL	gBFDMA		; set DMA address to next block
		EX	DE,HL		; HL= read address
		LD	DE,(gvDFWC)	; DE= FCB address
		CALL	gBFWRR		; random write does not inc rec nbr
		PUSH	HL		; save read address
		LD	HL,(gvDFWC)	; HL= FCB address
		LD	DE,gcFCRR	; DE= offset to record number
		ADD	HL,DE		; HL= record number address
		INC	(HL)		; bump lobyte (r0)
		JR	NZ,gDFWRD	; no overflow, done
		INC	HL		; overflow
		INC	(HL)		; bump hibyte (r1)
gDFWRD:		POP	HL		; recall read address		

gDFWD:		OR	A		; is write result zero?
		LD	A,gcDFWF	; preset to error
		JR	NZ,gDFWE	; no, error

		LD	DE,128		; DE= block size
		ADD	HL,DE		; add it to read address
		JR	gDFWL		; go for next block

;--------------------------------------------------------------- error handler
gDFWE:		SCF			; signal error to caller
		JR	gDFWX		; and exit
		
;--------------------------------------------------------------- no-error exit
gDFWQ:		XOR	A		; CY=0, signal no error
	
;--------------------------------------------------------- close file and exit
; The file needs to be closed after every write operation, otherwise there
; will be holes in the block allocation table caused by the write random
; function, making the file unreadable.
;-----------------------------------------------------------------------------
gDFWX:		PUSH	AF
		LD	DE,(gvDFWC)	; DE= FCB address
		CALL	gDFCL		; close the file, clean up FCB
		POP	AF

		POP	BC
		POP	DE
		POP	HL
		RET

;------------------------------------------------------------------- variables
gvDFWC:		DW	0000H		; FCB address	
;================================================================== eoF gDFW()
		ENDIF			; ugDFW

;+++++++++++++++++++++++++++++++++++++++++++++++++++++++++ disk I/O using BIOS
		IF	ugDSPT
;===================================================================== gDSPT()
; Purpose.....:	Read number of sectors per track out of DiskParameterBlock
;		of the currently selected drive, select drive in C
; Input.......: C:  drive to use (A=0, B=1, ...)
; Output......:	A:  number of sectors per track
;		CY: 0= success
;		    1= invalid drive in C on call
; Registers...: AF destroyed
; Requires....:	gBIOS()
; Labels......:	gDSPTx: g)enp D)isk get S)ectors P)er T)rack
;==============================================================================
gDSPT::		PUSH	HL
		PUSH	DE
		PUSH	BC

		LD	A,gcISDR	; SELDSK function
		CALL	gBIOS		; get DPB address in HL

		LD	A,L		; SELDSK returns HL=0
		OR	H		; if a non-existent drive
		SCF			; was given
		JR	Z,gDSPTX	; return with CY=1

		LD	DE,0AH		; offset in DPH to DPB addr
		ADD	HL,DE		; HL=DPB address
		LD	E,(HL)		; DE=(HL)
		INC	HL
		LD	D,(HL)
		LD	A,(DE)		; get sectors per track (SPT)
		SCF			; clear CY
		CCF			; to signal success

gDSPTX:		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================= eoF gDSPT()
		ENDIF
		IF	ugDRW
;====================================================================== gDRW()
; Purpose.....:	Read/Write sectors from/to disk via BIOS
;		Saves and restores the logged disk. This cannot be done
;		using BDOS (by caller) if it was changed via BIOS (here). 
; Input.......:	HL: Buffer address, if 0, use last used address
;		D:  Starting track number
;		E:  Starting sector number
;		B:  Number of sectors to read/write
;		C:  Drive number (A=0, B=1)
;		CY: 0= read, 1=write
; Output......:	CY: 0= ok, no error, 1= some error
;		A:  if error, one of gcDRWD or gcDRWI
;		D:  if I/O error, track number
;		E:  if I/O error, sector number
;		HL: points to end of buffer, undefined if error 
; Registers...: All destroyed
; Requires....: gBIOS()
; Labels......: gDRWx: g)enp D)isk R)ead W)rite
;==============================================================================
gDRW::		PUSH	AF		; function (read/write)

		PUSH	BC		; BC is destroyed by BDOS call
		CALL	gBRCD		; get logged disk for restore on exit
		LD	(gvDRWL),A	; save it
		POP	BC

		LD	A,H		; check if HL = 0
		OR	L		; if so, use last DMA address
		JR	Z,gDRWND
		LD	(gvDRWD),HL	; use new DMA address

gDRWND:		LD	H,00H
		LD	L,D
		LD	(gvDRWT),HL	; actual track
		LD	L,E
		LD	(gvDRWS),HL	; actual sector
		LD	A,B
		LD	(gvDRWC),A	; sector counter

		CALL	gDSPT		; select drive, get sectors per track
		JR	C,gDRWED	; invalid drive
		LD	(gvDRWP),A

gDRWL:		LD	BC,(gvDRWT)	; BC=actual tack
		LD	A,gcISTR	; SETTRK: set track
		CALL	gBIOS

		LD	BC,(gvDRWS)	; BC=actual sector
		LD	A,gcISSE	; SETSEC: set sector
		CALL	gBIOS

		LD	BC,(gvDRWD)	; BC=actual DMA address
		LD	A,gcISDM	; SETDMA: set DMA address
		CALL	gBIOS
		LD	HL,(gvDRWD)	; add 128 to DMA address
		LD	DE,128
		ADD	HL,DE
		LD	(gvDRWD),HL

		POP	AF		; get read/write mode
		PUSH	AF		; correct stack
		JR	C,gDRWW		; write mode, go and write
		LD	A,gcIRDS	; READ: read a sector
		JR	gDRWRW		; go and read

gDRWW:		LD	A,gcIWRS	; WRITE: write a sector
gDRWRW:		CALL	gBIOS
		CP	0		; check for error
		JR	NZ,gDRWEI	; we have an error

		LD	A,(gvDRWC)	; get sector counter
		DEC	A		; decrement it
		LD	(gvDRWC),A	; and write back
		OR	A		; all done?
		JR	NZ,gDRWNS	; no, next sector
		INC	A		; clear Z flag, no error
		JR	gDRWX		; and exit

gDRWNS:		LD	HL,gvDRWS
		INC	(HL)		; next sector for SETSEC

		LD	A,(gvDRWP)	; check if all sects per track
		CP	(HL)		; are read. If so, inc track
		JR	NZ,gDRWL

		LD	HL,gvDRWT	; set next track number
		INC	(HL)
		XOR	A		; restart with sector 0
		LD	(gvDRWS),A
		JR	gDRWL

gDRWED:		SCF			; CY=1: error
		LD	A,gcDRWD	; code for drive select error
		JR	gDRWX		; drive select error

gDRWEI:		LD	A,(gvDRWT)	; i/o error
		LD	D,A		; D=track number
		LD	A,(gvDRWS)
		LD	E,A		; E=sector number
		SCF			; CY=1: error
		LD	A,gcDRWI	; code for I/O error

gDRWX:		PUSH	AF		; save return status
		LD	A,(gvDRWL)	; get logged-in disk
		LD	C,A		; gDSPT() wants drive in C
		CALL	gDSPT		; restore to value on entry
		POP	AF

		INC	SP		; POP AF may not be used
		INC	SP		; flags return status
		RET

;------------------------------------------------------------- local variables
gvDRWD:		DW	0		; current DMA address
gvDRWT:		DW	0		; current track number
gvDRWS:		DW	0		; current sector number
gvDRWP:		DB	0		; sectors per track
gvDRWC:		DB	0		; sector counter
gvDRWL:		DB	0		; logged-in disk save

;----------------------------------------------------------- errors definition
		PUBLIC	gcDRWD
		PUBLIC	gcDRWI
gcDRWD		EQU	0FFH		; drive select error
gcDRWI		EQU	0FEH		; i/o error
;================================================================== eoF gDRW()
		ENDIF			; ugDRW
;############################################################### EOF LIBGD.INC
=============== eoF gDRW()
		ENDIF			; ugDRW
;########################################