;################################################################### LIBGO.INC
; Purpose.....:	LIBG: Conversion functions, included by LIBG.MAC
;#############################################################################
		IF	ugOUC
;====================================================================== gOUC()
; Purpose.....: Convert letter to uppercase
; Input.......: A: Letter to convert
; Output......: A: Converted letter
; Registers...: A destroyed
; Requires....: Nothing
; Labels......: gOUCxx: g)enp cO)nvert to U)pperC)ase
;=============================================================================
gOUC::		CP	'a'		; >= 'a'?
		JR	C,gOUCX		; no, nothing to do
		CP	'z'+1		; >= 'z'?
		JR	NC,gOUCX	; yes, nothing to do

		SUB	020H		; convert to uppercase

gOUCX:		RET
;================================================================== eoF gOUC()
		ENDIF
		IF	ugOBD
;====================================================================== gOBD()
; Purpose.....: Convert binary to decimal (0..65535)
; Input.......: DE: Value to convert
;               HL: Where to store result
;               C : Number of digits to convert (1..5)
;		CY=1: leading zeros
;		CY=0: leading blanks
; Output......: (HL): Converted ASCII string
; Registers...: All saved
; Requires....: Nothing
; Labels......: gOBDxx: g)enp cO)nvert B)inary to D)ecimal 
;=============================================================================
gOBD::		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF
		PUSH	IX

		LD	B,0		; B= flags
		RL	B		; B0= leading zeros/blanks	

		LD	A,5		; check max digits
		CP	C
		JR	NC,gOBDG
		LD	C,5		; >5, make 5

gOBDG:		PUSH	HL
		POP	IX		; IX=result buffer address
		EX	DE,HL		; HL=value to convert
		LD	A,C		; A=number of digits
		
		LD	DE,10000	; digit 5
		CALL	gUDIV		; HL=modulo, DE=div result
		CP	5		; convert 10'000s?
		CALL	NC,gOBDS	; yes, convert & store

		LD	DE,1000		; digit 4
		CALL	gUDIV		; HL=modulo, DE=div result
		CP	4		; convert 1'000s?
		CALL	NC,gOBDS	; yes, convert and store
		
		LD	DE,100		; digit 3
		CALL	gUDIV		; HL=modulo, DE=div result
		CP	3		; convert 100s?
		CALL	NC,gOBDS	; yes, convert and store

		LD	DE,10		; digit 2
		CALL	gUDIV		; HL=modulo, DE=div result
		CP	2		; convert 10s?
		CALL	NC,gOBDS	; yes, convert and store

		LD	E,L		; get remainder
		SET	1,B		; it's the last digit, show the '0'
		CALL	gOBDS		; digit 1, convert and store

gOBDX:		POP	IX
		POP	AF
		POP	BC
		POP	DE
		POP	HL
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ gOBDS()
; Purpose.....:	Convert to ASCII and store in buffer
; Input.......: A= value to convert
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gOBDS:		PUSH	AF
		LD	A,E		; A= binary value
		ADD	A,'0'		; convert to ASCII
		BIT	0,B		; show leading zeros?
		JR	NZ,gOBDSW	; yes, no further checks
		BIT	1,B		; no, last digit?
		JR	NZ,gOBDSW	; yes, show '0' if zero
		BIT	2,B		; was there a non-zero yet?
		JR	NZ,gOBDSW	; yes, zero as '0'
		CP	'0'		; first non-zero?
		JR	Z,gOBDSS	; no, use space
		SET	2,B		; 1st non-zero digit
		JR	gOBDSW		; store it

gOBDSS:		LD	A,gcSPC		; use a space instead of '0'

gOBDSW:		LD	(IX+0),A
		INC	IX
		POP	AF
		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoLF gOBDS()
;================================================================== eoF gOBD()
		ENDIF
		IF	ugODB
;====================================================================== gODB()
; Purpose.....: Convert decimal ASCII to binary (0..65535)
;		Conversion stops at the 1st non-numeric or C gets zero
; Input.......: DE: start of ASCII digits
;               C : Number of digits to convert (1..5)
; Output......: HL: Binary value
;		CY=1 if overflow or number of digits =0
; Registers...: AF, HL destroyed
; Requires....: Nothing
; Labels......: gODBxx: g)enp cO)nvert D)ecimal to B)inary 
;=============================================================================
gODB::		PUSH	DE
		PUSH	BC

		XOR	A
		OR	C
		JR	Z,gODBX		; digits=0, exit with CY=1
 
		LD	HL,0		; init result

gODBL:		LD	A,(DE)		; get digit
		INC	DE		; point to next
		CP	gcSPC		; space equals to '0'
		JR	NZ,gODBC	; not space
		LD	A,'0'		; space, make '0'	

gODBC:		CP	'9'+1		; check numeric range
		JR	NC,gODBX	; >'9', abort
		CP	'0'
		JR	C,gODBX		; <'0', abort

		SUB	'0'		; convert to binary
		CALL	gODBM		; multiply by 10 and add A
		JR	C,gODBX		; overflow, exit
		DEC	C		; digits -1
		JR	NZ,gODBL	; go for next

gODBX:		POP	BC
		POP	DE
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ gODBM()
; Purpose.....:	Multiply HL by 9, add A
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gODBM:		PUSH	DE

		LD	D,H		; DE=HL
		LD	E,L
		LD	B,9		; loop counter

gODBML:		ADD	HL,DE		; add ten times
		JR	C,gODBMX	; overflow
		DJNZ	gODBML

		LD	D,0		
		LD	E,A		; add new digit to result
		ADD	HL,DE

gODBMX:		POP	DE
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoF gODBM()
;================================================================== eoF gODB()
		ENDIF			; ugODB
		IF	ugOBH
;====================================================================== gOBH()
; Purpose.....: Convert binary to hex (0..FFFF), leading zeros always shown
; Input.......: DE: Value to convert
;               HL: Where to store result
;               C : Number of digits to convert (1..4)
; Output......: (HL): Converted ASCII string
; Registers...: All saved
; Requires....: gSTRM() for getting terminator
; Labels......: gOBHxx: g)enp cO)nvert B)inary to H)ex 
;=============================================================================
gOBH::		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF

		LD	A,4		; check max digits
		CP	C
		JR	NC,gOBH4
		LD	C,4		; >4, make 4

gOBH4:		LD	A,C		; convert 4th digit?
		CP	4
		JR	C,gOBH3		; no, try 3rd
		LD	A,D
		CALL	gOBHH		; local sub: convert hi nibble

gOBH3:		LD	A,C
		CP	3		; convert 3rd digit?
		JR	C,gOBH2		; no, try 2nd
		LD	A,D
		CALL	gOBHL		; local sub: convert lo nibble

gOBH2:		LD	A,C
		CP	2		; convert 2nd digit?
		JR	C,gOBH1		; no, do 1st
		LD	A,E
		CALL	gOBHH		; local sub: convert 2nd digit

gOBH1:		LD	A,E
		CALL	gOBHL		; local sub: convert 1st digit

gOBHX:		POP	AF
		POP	BC
		POP	DE
		POP	HL
		RET

;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ gOBHx()
; Purpose.....:	Convert and store hi/lo nibble to HEX-ASCII
; Input.......: A= value to convert gOBHH(): hi nibble, gOBHL(): lo nibble
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~
gOBHH:		SRL	A		; shift to low nibble
		SRL	A
		SRL	A
		SRL	A

gOBHL:		AND	0FH		; mask hi nibble
		ADD	A,'0'		; convert to ASCII
		CP	'9'+1		; decimal range?
		JR	C,gOBHS		; yes, store
		ADD	A,7		; make 'A'..'F'

gOBHS:		LD	(HL),A
		INC	HL
		RET
;~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~ eoLF gOBHx()
;================================================================== eoF gOBH()
		ENDIF
		IF	ugOHB
;====================================================================== gOHB()
; Purpose.....: Convert HEX ASCII to binary (0..65535)
;		Conversion stops at the 1st non-HEX or C gets zero
; Input.......: DE: start of ASCII digits
;               C : Number of digits to convert (1..4)
; Output......: HL: Binary value
;		CY=1 if overflow or number of digits=0
; Registers...: AF, HL destroyed
; Requires....: Nothing
; Labels......: gOHBxx: g)enp cO)nvert H)ex to B)inary 
;=============================================================================
gOHB::		PUSH	DE
		PUSH	BC

		XOR	A		; check digits >0
		OR	C
		JR	Z,gOHBX		; zero, exit with CY=1

		LD	HL,0		; init result

gOHBL:		LD	A,(DE)		; get digit
		INC	DE		; point to next
		CALL	gOUC		; convert to uppercase
		CP	gcSPC		; space equals to '0'
		JR	NZ,gOHBC	; not space
		LD	A,'0'		; space, make '0'	

gOHBC:		CP	'0'		; check HEX range
		JR	C,gOHBX		; <'0', abort
		CP	'9'+1	
		JR	C,gOHBS		; <='9', convert
		CP	'F'+1
		JR	NC,gOHBX	; >'F', abort
		CP	'A'
		JR	C,gOHBX		; <'A', abort
		SUB	7		; it's A..F, skip gap

gOHBS:		SUB	'0'		; convert to binary
		LD	B,4		; shift HL left 4 times
gOHBSL:		SLA	L		; multiply by 16
		RL	H
		DJNZ	gOHBSL
		JR	C,gOHBX		; overflow, exit with CY=1

		OR	L		; add the new digit
		LD	L,A
		DEC	C		; digits -1
		JR	NZ,gOHBL	; go for next
		XOR	A		; success, exit with CY=0

gOHBX:		POP	BC
		POP	DE
		RET
;================================================================== eoF gOHB()
		ENDIF			; ugOHB
;############################################################### EOF LIBGO.INC
