;################################################################### LIBGU.INC
; Purpose.....:	LIBG: Utility functions, included by LIBG.MAC
;#############################################################################
		IF	ugUDLY
;===================================================================== gUDLY()
; Purpose.....: Delay for given number of milliseconds
;               Uses xxx machine cycles for one loop (BC=1)
; Input.......: BC=Number of milliseconds
; Registers...: BC destroyed
; Requires....: Nothing
; Labels......: gUDLYx: g)enp U)tility D)eL)aY)
;=============================================================================
gUDLY::		PUSH	BC
		PUSH	DE
		PUSH	AF

;------------------------------------------------------- DE * 70 cycles per BC
gUDLY1:		LD	A,B		; 7cc * BC
		OR	C		; 4cc * BC
		JR	Z,gUDLYX	; 7cc * BC

;-------------------------------------------- delay one ms, 52 cycles per DE-1
		LD	DE,hcCCMS/31	; 10cc * BC

;---------------------------------------------------- 24 clock cycles per loop
gUDLY2:		DEC	DE		; 6cc * BC * DE
		LD	A,D		; 7cc * BC * DE
		OR	E		; 4cc * BC * DE
		JR	NZ,gUDLY2	; 7cc * BC * DE

		DEC	BC		; 6cc * BC
		JR	gUDLY1		; 12cc * BC

gUDLYX:		POP	AF
		POP	DE
		POP	BC
		RET
;================================================================= eoF gUDLY()
		ENDIF
		IF	ugUMF
;====================================================================== gUMF()
; Purpose.....: Fill memory area with constant
; Input.......: A : Fill byte
;               HL: Starting address
;               BC: Number of bytes to write
; Registers...: All saved
; Requires....: Nothing
; Labels......: gUMFxx: g)enp U)tility M)emory F)ill
;=============================================================================
gUMF::		PUSH	HL
		PUSH	DE
		PUSH	BC
		PUSH	AF

		LD	E,A		; fillbyte to E

gUMFL:		LD	A,B		; check if done
		OR	C
		JR	Z,gUMFX		; yes, exit
		LD	(HL),E		; write next
		INC	HL		; pump pointer
		DEC	BC		; dec counter
		JR	gUMFL

gUMFX:		POP	AF
		POP	BC
		POP	DE
		POP	HL
		RET
;================================================================== eoF gUMF()
		ENDIF			; ugUMF
		IF	ugUHLDE
;==================================================================== gUHLDE()
; Purpose.....: Compare HL and DE
; Input.......: HL, DE to check
; Output......: SBC HL,DE flags
; Registers...: F destroyed
; Requires....: Nothing
; Labels......: gUHLDE: g)genp U)tility HL) DE)
;=============================================================================
gUHLDE::	PUSH	HL
		PUSH	DE

		SCF			; make sure carry is clear
		CCF
		SBC	HL,DE		; compare the two
	
		POP	DE
		POP	HL
		RET
;================================================================ eoF gUHLDE()
		ENDIF
		IF	ugUDIV
;===================================================================== gUDIV()
; Purpose.....: Divide and get modulo
; Input.......: HL: Dividend
;               DE: Divisor
; Output......: HL: HL % DE, -1 if divisor = zero
;               DE: HL / DE
; Registers...: HL,DE Destroyed
; Requires....: Nothing
; Labels......: gODIVx: g)enp U)titity DIV)ide
;=============================================================================
gUDIV::		PUSH	AF
		PUSH	BC

		LD	BC,0FFFFH	; preset division result

		LD	A,D		; prevent division by zero,
		OR	E		; would give a infinite loop
		JR	Z,gUDIVX	; exit if zero

gUDIVL:		INC	BC		; bump result
		XOR	A		; clear carry
		SBC	HL,DE		; subtract until
		JR	NC,gUDIVL	; overflow occurs

		ADD	HL,DE		; overflow occurred, restore last	
		LD	D,B		; division result
		LD	E,C		; to DE
		
gUDIVX:		POP	BC
		POP	AF
		RET
;================================================================= eoF gUDIV()
		ENDIF
;############################################################### EOF LIBGU.INC
======================== eoF gUDIV()
		ENDIF
;############################################################