;################################################################### LIBHK.INC
; Purpose.....:	LIBH: Direct keyboard functions
; History.....: 08.01.2024: Created
;#############################################################################

;======================================================================= hKR()
; Purpose.....:	Check if a key is ready from the keyboard. Returns it if
;		one is ready. Uses MOS calls, is slightly faster than BDOS.
; Output......: A   : code of pressed key, 0 if no key pressed
;		Z=1 : no key pressed
;		Z=0 : key pressed, code in A
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hKRxxx: h)ardware K)eyboard R)ead
; Remarks.....: Call addresses taken from 'ITT-3030_MOS-Dokumentation'
;=============================================================================
hKR::		PUSH	HL
		CALL	0FE12H		; read keyboard status using MOS
		JR	NZ,hKRR		; key pressed, read it
		XOR	A		; A= 0 for no key 
		JR	hKRX

hKRR:		CALL	0FE73H		; wait for key using MOS
		OR	A		; clear Z flag

hKRX:		POP	HL
		RET
;=================================================================== eoF hKR()

;############################################################### EOF LIBHK.INC
================================