;################################################################## LIBHP1.INC
; Purpose.....:	LIBH, PIO1 functions. Included by LIBH.MAC
;#############################################################################
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ PIO1 channel A
		IF	uhP1A OR uhP1P
;===================================================================== hP1AS()
; Purpose.....:	Initialize PIO1, channel A
; Input.......: A: Mode, 0..3
;		B: Mode3: in/out configuration, high=input  
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hP1ASxx: h)ardware P)IO 1) channel A) S)etup
;=============================================================================
hP1AS::		RRCA			; mode bits to bit 7,6
		RRCA
		OR	0FH		; set mode command
		OUT	(hcP1AC),A	; write to CH-A control port
		CP	0C0H		; mode3?
		JR	NZ,hP1ASX	; no, exit
		LD	A,B		; get in/out bit setting
		OUT	(hcP1AC),A	; and write it
		
hP1ASX:		RET
;================================================================= eoF hP1AS()
		ENDIF
		IF	uhP1A
;==================================================================== hP1AIO()
; Purpose.....:	Read/write data to/from PIO1 channel-A
; Input.......: A: Data to write
; Output......: A: Data read
;		CY=1: Write data, CY=0: Skip write operation
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hP1AIO: h)ardware P)IO 1) channel A) I)nput O)utput
;=============================================================================
hP1AIO::	JR	NC,hP1AIR
		OUT	(hcP1AD),A	; write to data port
hP1AIR:		IN	A,(hcP1AD)	; read from port
		RET
;================================================================ eoF hP1AIO()
		ENDIF	
;++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++ PIO1 channel B
		IF	uhP1B OR uhP1P
;===================================================================== hP1BS()
; Purpose.....:	Initialize PIO1, channel B
; Input.......: A: Mode, 0..3
;		B: Mode3: in/out configuration, high=input  
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hPBSxx: h)ardware P)IO 1) channel B) S)etup
;=============================================================================
hP1BS::		RRCA			; mode bits to bit 7,6
		RRCA
		OR	0FH		; set mode command
		OUT	(hcP1BC),A	; write to CH-B control port
		CP	0C0H		; mode3?
		JR	NZ,hP1BSX	; no, exit
		LD	A,B		; get in/out bit setting
		OUT	(hcP1BC),A	; and write it
		
hP1BSX:		RET
;================================================================= eoF hP1BS()
		ENDIF
		IF	uhP1B
;==================================================================== hP1BIO()
; Purpose.....:	Read/write data to/from PIO1 channel-B
; Input.......: A: Data to write
; Output......: A: Data read
;		CY=1: Write data, CY=0: Skip write operation
; Registers...: AF destroyed
; Requires....: Nothing
; Labels......: hP1BIO: h)ardware P)IO 1) channel B) I)nput O)utput
;=============================================================================
hP1BIO::	JR	NC,hP1BIR
		OUT	(hcP1BD),A	; write to data port
hP1BIR:		IN	A,(hcP1BD)	; read from port
		RET
;================================================================ eoF hP1BIO()
		ENDIF
;+++++++++++++++++++++++++++++++++++++++++++++++++++++++ PIO1 parallel printer
		IF	uhP1P
;===================================================================== hP1PS()
; Purpose.....: Initalize PIO1 for printer use
; Registers...: All saved
; Requires....:	hP1AS(), hP1BS()
; Labels......: hP1PSx: h)ardware P)IO 1) lineP)rinter S)etup
;=============================================================================
hP1PS::		PUSH	AF

		LD	A,hcPMOU	; port A output mode (printer data)
		CALL	hP1AS		; init port A
		LD	A,hcPMBC	; port B control mode (busy/strobe)
		LD	B,00010000B	; bit1=busy, bit4=strobe
		CALL	hP1BS		; init port B

		POP	AF
		RET
;================================================================= eoF hP1PS()

;===================================================================== hP1PW()
; Purpose.....: Check printer status, write byte to printer
; Input.......: A: byte to send
; Output......: CY=1: abort was pressed while waiting for printer ready
;               Z=1:  printer not ready if no-wait was specified
;               Z=0:  data sent
; Registers...: AF destroyed
; Requires....:	Nothing
; Labels......: hP1Pxx: h)ardware P)IO 1) lineP)rinter W)rite
;=============================================================================
;---------------------------------------------------- port and bit definitions
hcP1WD		EQU	hcP1AD		; printer data port (CH-A)
hcP1WC		EQU	hcP1BD		; printer control port (CH-B)

hcP1WR		EQU	1		; printer ready bit (CH-B, bit1)
hcP1WS		EQU	4		; printer strobe bit (CH-B, bit4)

hP1PW::		PUSH	BC
		LD	B,A		; save char to send

		RES	7,C		; make sure abort key flag is zero
		LD	A,(hvP1KA)	; get abort key
		OR	A		; abort enabled?
		JR	Z,hP1PWL	; no
		LD	C,A		; save abort key
		SET	7,C		; set abort flag for faster polling	

;------------------------------------------ check keyboard for abort character
hP1PWL:		BIT	7,C		; shall we wait until ready?
		JR	NZ,hP1PWC	; no, skip keyboard check
		CALL	hKR		; get keyboard char 
		SET	7,A
		CP	C		; abort key pressed?
		LD	A,B		; recall original A
		SCF			; preset to ctl-c pressed
		JR	Z,hP1PWX	; abort key pressed, exit with CY=1 

;-------------------------------------------------- check printer ready status
hP1PWC:		IN	A,(hcP1WC)	; get control port data
		BIT	hcP1WR,A	; check for ready bit active
		JR	NZ,hP1PWW	; printer is ready, go and print

;-------------------------------------------------------- printer is not ready
		BIT	7,C		; shall we wait for ready?
		JR	Z,hP1PWL	; yes
		XOR	A		; no
		JR	hP1PWX		; exit with Z=1

;-------------------------------------------------- write data to printer port
hP1PWW:		LD	A,B		; recall data byte	
		OUT	(hcP1WD),A	; write to data port
	
;---------------------------------------------------------- apply strobe pulse
hP1PWS:		IN	A,(hcP1WC)	; get control port data again
		SET	hcP1WS,A	; set strobe bit
		OUT	(hcP1WC),A	; write to printer
		NOP			; short delay
		NOP
		RES	hcP1WS,A	; reset strobe bit
		OUT	(hcP1WC),A	; write to printer
		XOR	A		; clear Z to signal
		INC	A		; byte was sent

hP1PWX:		POP	BC
		RET
;================================================================= eoF hP1PW()

;==================================================================== hP1PKS()
; Purpose.....: Set abort key for printing
; Input.......: A: Abort character, 0= no keyboard check
; Registers...: All saved
; Requires....: Nothing
; Labels......: hP1PKSx: h)ardware P)IO 1) lineP)rinter abortK)ey S)et
;=============================================================================
hP1PKS::	LD	(hvP1KA),A
		RET

;------------------------------------------------------------------ local data
hvP1KA:		DB	0		; print abort key
;================================================================= eoF hPPKS()
		ENDIF

;############################################################## EOF LIBHP1.INC
====================== eoF hPPKS()
		ENDIF

;#########################################################